""" Tests for superai.utils.stopwatch """

from superai.utils import stopwatch
from superai.log import logger
import logging

@stopwatch
def fibonacci(n):
    """ Fibonacci series

    0, 1, 1, 2, 3, 5, 8, 13, ...
    """
    if n <= 1:
        return n
    a = 0
    b = 1
    c = 1
    for i in range(2, n):
        a = b
        b = c
        c = a + b
    return c


def test_stopwatch(caplog):
    """ Basic stopwatch test """
    func = fibonacci
    with caplog.at_level(logging.DEBUG):
        func(100)

    assert f'{__name__}' in caplog.text, "Module name is logged using stopwatch"
    assert f'{fibonacci.__name__}()' in caplog.text, "Function name is logged using stopwatch"
    assert f'elapsed time: ' in caplog.text, "Elapsed time logged using stopwatch"



from os import makedirs, rename
from pathlib import Path
from random import random
from typing import Optional

import pytest
import yaml

from superai.config import (
    __env_switcher,
    __local_settings_path,
    __secrets_path,
    __settings_path,
    add_secret_settings,
    get_config_dir,
    list_env_configs,
    set_env_config,
)

base_dir = Path(get_config_dir()).expanduser()


def _mv_file(src: Path, dst: Path = None) -> Optional[Path]:
    # Nothing to do
    if not src:
        return

    file_path = Path(src).expanduser()
    tmp_file_path = Path(dst).expanduser() if dst else Path(f"{file_path.absolute()}.{random()}")

    # Nothing to do
    if not file_path.exists():
        return None

    # # Do not override existing tmp file since that indicates that a previous test didn't clean properly
    if tmp_file_path.exists():
        raise ValueError(f"tmp_path {dst} exists already")

    rename(file_path.absolute(), tmp_file_path.absolute())

    return tmp_file_path


def _setup_file(file_path: Path) -> Path:
    makedirs(base_dir.absolute(), exist_ok=True)
    f_path = Path(file_path).expanduser().absolute()
    tmp = _mv_file(f_path)

    if not f_path.exists():
        f_path.touch()
    return tmp


@pytest.fixture()
def with_secrets_file():
    s_path = Path(__secrets_path).expanduser().absolute()
    tmp = _setup_file(s_path)
    yield str(s_path.absolute())
    s_path.unlink()
    if tmp:
        _mv_file(tmp, s_path)
        assert s_path.exists()


@pytest.fixture()
def with_settings_file():
    s_path = Path(__settings_path).expanduser().absolute()
    tmp = _setup_file(s_path)
    yield str(s_path.absolute())
    s_path.unlink()
    if tmp:
        _mv_file(tmp, s_path)
        assert s_path.exists()


@pytest.fixture()
def with_dotenv_file():
    dot_env_path = base_dir / ".env"
    s_path = Path(dot_env_path).expanduser().absolute()
    tmp = _setup_file(s_path)
    yield str(s_path.absolute())
    s_path.unlink()
    if tmp:
        _mv_file(tmp, s_path)
        assert s_path.exists()


def test_root_dir_created():
    """
    If the init method is run, then the base folder should exist
    """
    assert base_dir.absolute().exists()


def test_file_is_not_modified_with_emtpy_content(with_secrets_file):
    secrets_path = Path(with_secrets_file).expanduser().absolute()
    with open(secrets_path, "r") as f:
        secrets_before = dict(yaml.load(f, yaml.SafeLoader) or {})
    content = None
    add_secret_settings(content)
    with open(secrets_path, "r") as f:
        secrets_after = dict(yaml.load(f, yaml.SafeLoader) or {})
    assert secrets_before == secrets_after


def test_new_content_is_added(with_secrets_file):
    secrets_path = Path(with_secrets_file).expanduser().absolute()
    content = {"my_test_content": "testing"}
    add_secret_settings(content)
    with open(secrets_path, "r") as f:
        secrets_after = dict(yaml.load(f, yaml.SafeLoader) or {})

    assert secrets_after.items() <= content.items()


def test_new_content_override(with_secrets_file):
    secrets_path = Path(with_secrets_file).expanduser().absolute()
    content = {"my_key": "testing"}
    add_secret_settings(content)
    with open(secrets_path, "r") as f:
        secrets_after = dict(yaml.load(f, yaml.SafeLoader) or {})
    assert secrets_after == content
    content = {"my_key": "modified"}
    add_secret_settings(content)
    with open(secrets_path, "r") as f:
        secrets_after = dict(yaml.load(f, yaml.SafeLoader) or {})
    assert secrets_after.items() <= content.items()


def test_list_envs_ignores(capsys):
    local_settings = Path(__local_settings_path).expanduser().absolute()
    with open(local_settings, "r") as f:
        settings_dict = dict(yaml.load(f, yaml.SafeLoader) or {})
    assert settings_dict["testing"] and settings_dict["default"]
    assert list_env_configs()["testing"]
    assert not list_env_configs().get("default")
    captured = capsys.readouterr()
    assert "- default" not in captured.out
    assert "- testing" not in captured.out
    assert "- sandbox" in captured.out
    assert "- prod" in captured.out


def test_set_env_success():
    dot_env_path = Path(base_dir / ".env").expanduser().absolute()
    set_env_config("testing")
    env = f"{__env_switcher}=testing"
    with open(dot_env_path, "r") as f:
        dot_env = f.readline()
    assert dot_env == env


@pytest.mark.xfail(ValueError, reason="Env config doesn't exist")
def test_set_env_fails_not_existent(with_dotenv_file):
    set_env_config("d")



from abc import ABC, abstractmethod
from typing import List


class AuthApiMixin(ABC):
    @abstractmethod
    def request(self, uri, method, body_params=None, query_params=None, required_auth_token=False):
        pass

    def get_apikeys(self) -> List[str]:
        """
        Get api-keys of authenticated user
        :return List with api-keys of authenticated user:
        """
        uri = "users/apiKeys"
        return self.request(uri, method="GET", required_auth_token=True)



import os
import warnings

from superai.config import add_secret_settings, get_config_dir, settings
from superai.log import logger

BASE_FOLDER = get_config_dir()

log = logger.get_logger(__name__)

_api_key_file = os.path.expanduser(f"{BASE_FOLDER}/apikey")


def save_api_key(api_key: str):
    _save_api_key_file(api_key)
    _save_api_key_secrets(api_key)


def _save_api_key_file(api_key: str):
    if not os.path.exists(os.path.dirname(_api_key_file)):
        log.debug(f"Creating path {os.path.dirname(_api_key_file)}")
        os.makedirs(os.path.dirname(_api_key_file))
    with open(_api_key_file, "w") as f:
        f.write(api_key)


def _save_api_key_secrets(api_key: str):
    env = settings.current_env
    secret = {env: {"user": {"api_key": api_key}}}
    add_secret_settings(secret)
    log.info(f"Api key added to env {env}")


def load_api_key() -> str:
    # First load from env variable
    api_key = settings.get("user", {}).get("api_key")

    # Try to load from file (higher priority)
    try:
        with open(_api_key_file) as f:
            api_key = f.readline()
            log.debug(f"Using api_key from {_api_key_file}")
    except Exception as e:
        log.debug(
            f"Error loading api_key from file: {_api_key_file}. Using user__api_key from "
            f"{BASE_FOLDER}/.secrets.yaml?: {api_key is None}"
        )

    if not api_key:
        warnings.warn("Api key is not initialized. Run superai login --username <email> to retrieve your api key")

    return api_key



import click
import json
import signal
import sys
from botocore.exceptions import ClientError
from datetime import datetime
from pprint import pprint
from typing import List
from warrant import Cognito

from superai import __version__
from superai.client import Client
from superai.config import get_config_dir, list_env_configs, set_env_config, settings
from superai.log import logger
from superai.utils import load_api_key, save_api_key

BASE_FOLDER = get_config_dir()
COGNITO_USERPOOL_ID = settings.get("cognito", {}).get("userpool_id")
COGNITO_CLIENT_ID = settings.get("cognito", {}).get("client_id")
COGNITO_REGION = settings.get("cognito", {}).get("region")

log = logger.get_logger(__name__)


def _signal_handler(s, f):
    sys.exit(1)


@click.group()
def cli():
    pass


@cli.command()
@click.option("--verbose/--no-verbose", "-vvv", help="Verbose output", default=False)
def info(verbose):
    """ Print CLI Configuration """
    click.echo("=================")
    click.echo("Super.AI CLI Info:")
    click.echo("=================")
    click.echo("Version: {}".format(__version__))
    click.echo("Environment: {}".format(settings.current_env))
    if verbose:
        click.echo(pprint(settings.as_dict(env=settings.current_env)))


@cli.group()
@click.pass_context
def env(ctx):
    """
    super.AI Config operations
    """
    pass


@env.command(name="list")
@click.pass_context
def env_list(ctx):
    """

    :param ctx:
    :return:
    """
    list_env_configs(printInConsole=True)


@env.command(name="set")
@click.option("--api-key", help="Your super.AI API KEY", required=False)
@click.option("--environment", "-e", help="Set environment", required=False)
@click.pass_context
def env_set(ctx, api_key, environment):
    """
    Set configuration
    """
    if environment:
        set_env_config(name=environment)
    if api_key:
        save_api_key(api_key)


@cli.group()
@click.pass_context
def client(ctx):
    """
    super.AI API operations
    """
    api_key = ""
    try:
        api_key = load_api_key()
    except Exception as e:
        pass
    if len(api_key) == 0:
        print("User needs to login or set api key")
        exit()
    ctx.obj = {}
    ctx.obj["client"] = Client(api_key=api_key)


@client.command(name="create_jobs")
@click.option("--app_id", "-a", help="Application id", required=True)
@click.option("--callback_url", "-c", help="Callback URL for post when jobs finish")
@click.option("--inputs", "-i", help="Json list with inputs")
@click.option("--inputs_file", "-if", help="URL pointing to JSON file")
@click.pass_context
def create_jobs(ctx, app_id: str, callback_url: str, inputs: str, inputs_file: str):
    """
    Submit jobs
    """
    client = ctx.obj["client"]
    print("Submitting jobs")
    json_inputs = None
    if inputs is not None:
        try:
            json_inputs = json.loads(inputs)
        except:
            print("Couldn't read json inputs")
            exit()
    print(client.create_jobs(app_id, callback_url, json_inputs, inputs_file))


@client.command(name="fetch_job")
@click.option("--job_id", "-j", help="Job id", required=True)
@click.pass_context
def fetch_job(ctx, job_id: str):
    """
    Get Job given job id
    """
    client = ctx.obj["client"]
    print(f"Fetching job {job_id}")
    print(client.fetch_job(job_id))


@client.command(name="fetch_batches_job")
@click.option("--app_id", "-a", help="App id", required=True)
@click.pass_context
def fetch_batches_job(ctx, app_id: str):
    """
    Get not processed Batches given app id
    """
    client = ctx.obj["client"]
    print(f"Fetching batches {app_id}")
    print(client.fetch_batches_job(app_id))


@client.command(name="fetch_batch_job")
@click.option("--app_id", "-a", help="App id", required=True)
@click.option("--batch_id", "-b", help="Batch id", required=True)
@click.pass_context
def fetch_batch_job(ctx, app_id: str, batch_id: str):
    """
    Get Batch given app id and batch id
    """
    client = ctx.obj["client"]
    print(f"Fetching batch {app_id} {batch_id}")
    print(client.fetch_batch_job(app_id, batch_id))


@client.command(name="get_job_response")
@click.option("--job_id", "-j", help="Job id", required=True)
@click.pass_context
def get_job_response(ctx, job_id: str):
    """
    Get Job response given job id
    """
    client = ctx.obj["client"]
    print(f"Getting job response {job_id}")
    print(client.get_job_response(job_id))


@client.command(name="cancel_job")
@click.option("--job_id", "-j", help="Job id", required=True)
@click.pass_context
def cancel_job(ctx, job_id: str):
    """
    Cancel a job given job id. Only for jobs in SCHEDULED, IN_PROGRESS or SUSPENDED state.
    """
    client = ctx.obj["client"]
    print(f"Cancelling job {job_id}")
    print(client.cancel_job(job_id))


@client.command(name="list_jobs")
@click.option("--app_id", "-a", help="Application id", required=True)
@click.option("--page", "-p", help="Page number", type=int)
@click.option("--size", "-s", help="Size of page", type=int)
@click.option("--sort_by", "-sort", help="Job field to sort by", type=str, default="id", show_default=True)
@click.option(
    "--order_by",
    "-order",
    help="Sort direction (asc or desc)",
    type=click.Choice(["asc", "desc"]),
    default="asc",
    show_default=True,
)
@click.option(
    "--created_start_date",
    "-c0",
    help="Created start date",
    type=click.DateTime(formats=["%Y-%m-%dT%H:%M:%SZ", "%Y-%m-%d"]),
)
@click.option(
    "--created_end_date",
    "-c1",
    help="Created end date",
    type=click.DateTime(formats=["%Y-%m-%dT%H:%M:%SZ", "%Y-%m-%d"]),
)
@click.option(
    "--completed_start_date",
    "-e0",
    help="Completed start date",
    type=click.DateTime(formats=["%Y-%m-%dT%H:%M:%SZ", "%Y-%m-%d"]),
)
@click.option(
    "--completed_end_date",
    "-e1",
    help="Completed end date",
    type=click.DateTime(formats=["%Y-%m-%dT%H:%M:%SZ", "%Y-%m-%d"]),
)
@click.option(
    "--status_in",
    "-s_in",
    help="Status of jobs",
    multiple=True,
    type=click.Choice(["SCHEDULED", "IN_PROGRESS", "FAILED", "SUSPENDED", "CANCELED", "EXPIRED", "COMPLETED"]),
)
@click.pass_context
def list_jobs(
    ctx,
    app_id: str,
    page: int,
    size: int,
    sort_by: str,
    order_by: str,
    created_start_date: datetime,
    created_end_date: datetime,
    completed_start_date: datetime,
    completed_end_date: datetime,
    status_in: List[str] = None,
):
    """
    Get a paginated list of jobs (without response) given an application id
    """
    client = ctx.obj["client"]
    print(f"Fetching jobs per application {app_id}")
    if len(status_in) == 0:
        status_in = None
    print(
        client.list_jobs(
            app_id,
            page,
            size,
            sort_by,
            order_by,
            created_start_date,
            created_end_date,
            completed_start_date,
            completed_end_date,
            status_in,
        )
    )


@client.command(name="download_jobs")
@click.option("--app_id", "-a", help="Application id", required=True)
@click.option(
    "--created_start_date",
    "-c0",
    help="Created start date",
    type=click.DateTime(formats=["%Y-%m-%dT%H:%M:%SZ", "%Y-%m-%d"]),
)
@click.option(
    "--created_end_date",
    "-c1",
    help="Created end date",
    type=click.DateTime(formats=["%Y-%m-%dT%H:%M:%SZ", "%Y-%m-%d"]),
)
@click.option(
    "--completed_start_date",
    "-e0",
    help="Completed start date",
    type=click.DateTime(formats=["%Y-%m-%dT%H:%M:%SZ", "%Y-%m-%d"]),
)
@click.option(
    "--completed_end_date",
    "-e1",
    help="Completed end date",
    type=click.DateTime(formats=["%Y-%m-%dT%H:%M:%SZ", "%Y-%m-%d"]),
)
@click.option(
    "--status_in",
    "-s_in",
    help="Status of jobs",
    multiple=True,
    type=click.Choice(["SCHEDULED", "IN_PROGRESS", "FAILED", "SUSPENDED", "CANCELED", "EXPIRED", "COMPLETED"]),
)
@click.pass_context
def download_jobs(
    ctx,
    app_id: str,
    created_start_date: datetime,
    created_end_date: datetime,
    completed_start_date: datetime,
    completed_end_date: datetime,
    status_in: List[str] = None,
):
    """
    Trigger processing of job responses that is sent to customer email once is finished.
    """
    client = ctx.obj["client"]
    print(f"Triggering job responses processing per application {app_id}")
    if len(status_in) == 0:
        status_in = None
    print(
        client.download_jobs(
            app_id, created_start_date, created_end_date, completed_start_date, completed_end_date, status_in
        )
    )


@client.command(name="create_ground_truth")
@click.option("--app_id", "-a", help="Application id", required=True)
@click.option("--input_json", "-i", help="Input json of ground truth", required=True)
@click.option("--label", "-l", help="Label (or output) json of ground truth", required=True)
@click.option("--tag", "-t", help="Tag ground truth data")
@click.option("--metadata", "-m", help="Metadata json")
@click.pass_context
def create_ground_truth(
    ctx, app_id: str, input_json: str = None, label: str = None, tag: str = None, metadata: str = None
):
    """
    Submit fresh ground truth data
    """
    client = ctx.obj["client"]
    print("Submitting fresh ground truth data")
    input_dict = None
    metadata_dict = None
    label_dict = None
    if input_json is not None:
        try:
            input_dict = json.loads(input_json)
        except:
            print("Couldn't load input json of ground truth")
            exit()
    if metadata is not None:
        try:
            metadata_dict = json.loads(metadata)
        except:
            print("Couldn't load metadata json of ground truth")
            exit()
    if label is not None:
        try:
            label_dict = json.loads(label)
        except:
            print("Couldn't load label json of ground truth")
            exit()
    print(client.create_ground_truth(app_id, input_dict, label_dict, tag, metadata_dict))


@client.command(name="update_ground_truth")
@click.option("--ground_truth_data_id", "-g", help="Ground truth data id", required=True)
@click.option("--input_json", "-i", help="Input json of ground truth")
@click.option("--label", "-l", help="Label (or output) json of ground truth")
@click.option("--tag", "-t", help="Tag ground truth data")
@click.option("--metadata", "-m", help="Metadata json")
@click.pass_context
def update_ground_truth(
    ctx, ground_truth_data_id: str, input_json: str = None, label: str = None, tag: str = None, metadata: str = None
):
    """
    Update (patch) ground truth data
    """
    client = ctx.obj["client"]
    print(f"Updating ground truth data {ground_truth_data_id}")
    input_dict = None
    metadata_dict = None
    label_dict = None
    if input_json is not None:
        try:
            input_dict = json.loads(input_json)
        except:
            print("Couldn't load input json of ground truth")
            exit()
    if metadata is not None:
        try:
            metadata_dict = json.loads(metadata)
        except:
            print("Couldn't load metadata json of ground truth")
            exit()
    if label is not None:
        try:
            label_dict = json.loads(label)
        except:
            print("Couldn't load label json of ground truth")
            exit()
    print(client.update_ground_truth(ground_truth_data_id, input_dict, label_dict, tag, metadata_dict))


@client.command(name="list_ground_truth_data")
@click.option("--app_id", "-a", help="Application id", required=True)
@click.option("--page", "-p", help="Page number", type=int)
@click.option("--size", "-s", help="Size of page", type=int)
@click.pass_context
def list_ground_truth_data(ctx, app_id: str, page: int, size: int):
    """
    List all ground truth data for an application
    """
    client = ctx.obj["client"]
    print(f"Fetching ground truth data per application {app_id}")
    print(client.list_ground_truth_data(app_id, page, size))


@client.command(name="get_ground_truth_data")
@click.option("--ground_truth_data_id", "-g", help="Ground truth data id", required=True)
@click.pass_context
def get_ground_truth_data(ctx, ground_truth_data_id: str):
    """
    Fetch single ground truth data object
    """
    client = ctx.obj["client"]
    print(f"Fetching ground truth data {ground_truth_data_id}")
    print(client.get_ground_truth_data(ground_truth_data_id))


@client.command(name="delete_ground_truth_data")
@click.option("--ground_truth_data_id", "-g", help="Ground truth data id", required=True)
@click.pass_context
def delete_ground_truth_data(ctx, ground_truth_data_id: str):
    """
    Mark ground truth data as deleted
    """
    client = ctx.obj["client"]
    print(f"Deleting ground truth data {ground_truth_data_id}")
    print(client.delete_ground_truth_data(ground_truth_data_id))


@client.command(name="create_ground_truth_from_job")
@click.option("--app_id", "-a", help="Application id", required=True)
@click.option("-job_id", "-j", help="Job id", required=True)
@click.pass_context
def create_ground_truth_from_job(ctx, app_id: str, job_id: str):
    client = ctx.obj["client"]
    print(f"Converting job {job_id} to ground truth data")
    print(client.create_ground_truth_from_job(app_id, job_id))


@cli.command()
@click.option("--api-key", help="Your super.AI API KEY", required=True)
def config(api_key):
    """
    Set api key.
    """
    save_api_key(api_key)


@cli.command()
@click.option("--username", "-u", help="super.AI Username", required=True)
@click.option("--password", prompt=True, hide_input=True)
def login(username, password):
    """
    Use username and password to get super.AI api key.
    """
    user = Cognito(
        access_key="AKIAIOSFODNN7EXAMPLE",
        secret_key="wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY",
        user_pool_id=COGNITO_USERPOOL_ID,
        client_id=COGNITO_CLIENT_ID,
        user_pool_region=COGNITO_REGION,
        username=username,
    )
    try:
        user.authenticate(password)
    except ClientError as e:
        if (
            e.response["Error"]["Code"] == "UserNotFoundException"
            or e.response["Error"]["Code"] == "NotAuthorizedException"
        ):
            print("Incorrect username or password")
            return
        else:
            print(f"Unexpected error: {e}")
            return

    client = Client(auth_token=user.access_token)
    api_keys = client.get_apikeys()
    if len(api_keys) > 0:
        save_api_key(api_keys[0])
        print(f"Api key {api_keys[0]} was set")
    else:
        print(f"User {username} doesn't have any api keys")


@cli.command()
def logout():
    """
    Remove stored api key
    """
    save_api_key("")
    print("Stored api key was removed")


def main():
    signal.signal(signal.SIGINT, _signal_handler)
    sys.exit(cli())


if __name__ == "__main__":
    main()



import requests
from abc import ABC, abstractmethod
from typing import BinaryIO, Generator, List

from superai.exceptions import SuperAIStorageError


class DataApiMixin(ABC):
    _resource = "data"

    @abstractmethod
    def request(self, uri, method, body_params=None, query_params=None, required_api_key=False):
        pass

    @property
    def resource(self):
        return self._resource

    def list_data(
        self,
        data_ids: List[str] = None,
        paths: List[str] = None,
        recursive: bool = False,
        signedUrl: bool = False,
        secondsTtl: int = 600,
        page: int = None,
        size: int = None,
    ) -> dict:
        """
        Get a paginated list of datasets, that can be filtered using an array of ids xor and array of paths
        :param data_ids: Array of data ids
        :param paths: Array of paths
        :param recursive: Get all datasets from recursive path (only takes first path of array)
        :param signedUrl: Get signed url for each dataset
        :param secondsTtl: Time to live for signed url
        :param page: Page number [0..N]
        :param size: Size of page
        :return: Paginated list of datasets dicts
        """
        query_params = {}
        if data_ids is not None:
            query_params["dataId"] = data_ids
        elif paths is not None:
            query_params["path"] = paths
            query_params["recursive"] = recursive
        query_params["signedUrl"] = signedUrl
        if signedUrl:
            query_params["secondsTtl"] = secondsTtl
        if page is not None:
            query_params["page"] = page
        if size is not None:
            query_params["size"] = size
        return self.request(self.resource, method="GET", query_params=query_params, required_api_key=True)

    def get_all_data(
        self,
        data_ids: List[str] = None,
        paths: List[str] = None,
        recursive: bool = False,
        signedUrl: bool = False,
        secondsTtl: int = 600,
    ) -> Generator[dict, None, None]:
        """
        Generator that retrieves all data filtered using an array of ids xor and array of paths
        :param data_ids: Array of data ids
        :param paths: Array of paths
        :param recursive: Get all datasets from recursive path (only takes first path of array)
        :param signedUrl: Get signed url for each dataset
        :param secondsTtl: Time to live for signed url
        :return: Generator that yields complete list of dicts with data objects
        """
        page = 0
        paginated_data = {"last": False}
        while not paginated_data["last"]:
            paginated_data = self.list_data(
                data_ids=data_ids,
                paths=paths,
                recursive=recursive,
                signedUrl=signedUrl,
                secondsTtl=secondsTtl,
                page=page,
                size=500,
            )
            for d in paginated_data["content"]:
                yield d
            page = page + 1

    def get_signed_url(self, path: str, secondsTtl: int = 600) -> dict:
        """
        Get signed url for a dataset given its path
        :param path: Dataset's path
        :param secondsTtl: Time to live for signed url
        :return: Dict with signedUrl of dataset
        """
        uri = f"{self.resource}/url"
        return self.request(
            uri, method="GET", query_params={"path": path, "secondsTtl": secondsTtl}, required_api_key=True
        )

    def delete_data(self, path: str) -> dict:
        """
        Delete dataset given its path
        :param path: Dataset's path
        :return: Dict with details of deleted dataset
        """
        return self.request(self.resource, method="DELETE", query_params={"path": path}, required_api_key=True)

    def upload_data(self, path: str, description: str, mimeType: str, file: BinaryIO) -> dict:
        """
        Create/update a dataset given its path using file and mimeType
        :param path: Path of dataset
        :param description: Description of dataset
        :param mimeType: Type of file
        :param file: Binary File value
        :return: Dataset created/updated
        """
        dataset = self.request(
            self.resource,
            method="POST",
            query_params={"path": path, "description": description, "mimeType": mimeType, "uploadUrl": True},
            required_api_key=True,
        )
        try:
            resp = requests.put(dataset.pop("uploadUrl"), data=file.read())
            if resp.status_code == 200 or resp.status_code == 201:
                return dataset
            else:
                raise SuperAIStorageError(
                    f'File {str(file)} referenced by dataset {dataset["path"]} couldn\'t be uploaded to super.AI '
                    f"Storage"
                )
        except Exception as e:
            raise SuperAIStorageError(
                f'File {str(file)} referenced by dataset {dataset["path"]} couldn\'t be uploaded to super.AI Storage '
                f"Error: {str(e)}"
            )



from __future__ import absolute_import, division, print_function, unicode_literals

from . import decorator as logdecorator
from .logger import debug, DEBUG, error, ERROR, get_logger, info, INFO, init, warn, WARNING



import os
import pathlib
import warnings
from logging import Logger
from typing import Dict

import yaml
from dynaconf import Dynaconf, Validator
from jsonmerge import merge

from superai.log import logger

local = pathlib.Path(__file__).parent.absolute()

__env_switcher: str = "ENV_FOR_SUPERAI"
__superai_root_dir: str = os.getenv("SUPERAI_CONFIG_ROOT") or "~/.superai"
__settings_path: str = os.path.expanduser(f"{__superai_root_dir}/settings.yaml")
__secrets_path: str = os.path.expanduser(f"{__superai_root_dir}/.secrets.yaml")
__local_settings_path: str = os.path.expanduser(f"{local}/settings.yaml")  # Default settings file
__local_secrets_path: str = os.path.expanduser(f"{local}/.secrets.yaml")

dynaconf_setting_files = [
    __local_settings_path,
    __local_secrets_path,
    __settings_path,
    __secrets_path,
]


def _get_config_path(log: Logger = None):
    log = log or logger.get_logger(__name__)
    config_path = None
    for p in reversed(dynaconf_setting_files):
        if "secrets" in p:
            continue
        if os.path.exists(p):
            print("Reading configs from {}".format(p))
            config_path = p
            break
        else:
            log.debug(f"{p} does not exist")

    if not config_path:
        print("Error: No setting.yaml file found")

    return config_path


def get_config_dir():
    return __superai_root_dir


def list_env_configs(printInConsole=True, log: Logger = None) -> Dict:
    """ List all available clusters """
    log = log or logger.get_logger(__name__)

    import yaml

    __config_path__ = _get_config_path()

    if printInConsole:
        print("Available envs:")

    with open(os.path.expanduser(f"{__config_path__}"), "r") as f:
        envs = yaml.safe_load(f)
        envs.pop("default")
        if printInConsole:
            for config in list(envs):
                # Default and testing environments are not relevant thus hidden from the output
                if config not in ["testing"]:
                    print("- {}".format(config))

    return envs


def set_env_config(name, root_dir: str = __superai_root_dir, log: Logger = None):
    """ Set the active cluster name """
    # settings.setenv("other", silent=False)
    log = log or logger.get_logger(__name__)

    env_config = list_env_configs(printInConsole=False)
    if name not in env_config:
        warnings.warn(f"Error loading env {name} choose one of {list(env_config.keys())}")
        raise ValueError(f"Env {name} doesn't exists")

    log.info(f"Setting config : {name}")
    with open(os.path.expanduser(f"{root_dir}/.env"), "w") as f:
        f.write(f"ENV_FOR_SUPERAI={name}")


def add_secret_settings(content: dict = None):
    content = content or {}
    secrets_path = os.path.expanduser(__secrets_path)
    secrets_folder = os.path.dirname(__secrets_path)
    _log.debug(f"Secrets path {os.path.dirname(secrets_path)}")
    if not os.path.exists(secrets_folder):
        _log.debug(f"Creating secrets path {os.path.dirname(secrets_path)}")
        os.makedirs(secrets_folder, exist_ok=True)

    if not os.path.exists(secrets_path):
        _log.debug(f"Creating secrets file {secrets_path}")
        pathlib.Path(secrets_path).touch()

    _log.debug(f"Loading secrets file: {secrets_path}")
    with open(secrets_path, "r") as f:
        secrets = dict(yaml.load(f, yaml.SafeLoader) or {})

    final_secrets = merge(secrets, content)

    with open(secrets_path, "w") as f:
        yaml.dump(final_secrets, f, allow_unicode=True, default_flow_style=False)
    _log.debug(f"Final secrets {final_secrets}")


def init_config(
    root_dir: str = __superai_root_dir,
):
    # This is necessary so that the first time the user initializes the repository
    # dynaconf doesn't fail
    root_dir = os.path.expanduser(root_dir)
    if not os.path.exists(root_dir):
        os.mkdir(root_dir)

    dot_env_file = os.path.join(root_dir, ".env")
    if not os.path.exists(dot_env_file):
        set_env_config(name="prod")


init_config()

validators = [
    # Ensure some parameters exists (are required)
    Validator(
        "NAME",
        "AGENT",
        "AGENT.FILE",
        "AGENT.HOST",
        "AGENT.WEBSOCKET",
        "BACKEND",
        "BASE_FOLDER",
        "BASE_URL",
        "BUILD_MANIFEST",
        "COGNITO.CLIENT_ID",
        "COGNITO.REGION",
        "COGNITO.USERPOOL_ID",
        "DEPENDENCY_VERSION",
        "HATCHERY_BUILD_FOLDER",
        "MEMO_BUCKET",
        "PROJECT_ROOT",
        "S3_BUCKET",
        must_exist=True,
    ),
    # validate a value is eq in specific env
    Validator("NAME", eq="test", env="testing"),
    Validator("NAME", eq="dev", env="dev"),
    Validator("NAME", eq="local", env="local"),
    Validator("NAME", eq="sandbox", env="sandbox"),
    Validator("NAME", eq="prod", env="prod"),
]

settings = Dynaconf(
    envvar_prefix="SUPERAI",
    env_switcher=__env_switcher,
    settings_files=dynaconf_setting_files,
    environments=True,
    includes=[
        # If any .secrets.yaml exits, will have higher priority than the previously
        # loaded config files
    ],
    load_dotenv=True,
    root_path=os.path.expanduser(__superai_root_dir),
    validators=validators,
    merge_enabled=True,
)

_log = logger.init(
    filename=settings.get("log", {}).get("filename"),
    console=settings.get("log", {}).get("console"),
    log_level=settings.get("log", {}).get("level"),
    log_format=settings.get("log", {}).get("format"),
)



import logging
import time
from functools import wraps

from superai.log import logger


def retry(exceptions, tries=5, delay=1, backoff=2, logger=logging):
    """
    Retry calling the decorated function using an exponential backoff.

    Args:
        exceptions: The exception to check. may be a tuple of
            exceptions to check.
        tries: Number of times to try (not retry) before giving up.
        delay: Initial delay between retries in seconds.
        backoff: Backoff multiplier (e.g. value of 2 will double the delay
            each retry).
        logger: Logger to use. If None, print.
    """

    def deco_retry(f):
        @wraps(f)
        def f_retry(*args, **kwargs):
            mtries, mdelay = tries, delay
            while mtries > 1:
                try:
                    return f(*args, **kwargs)
                except exceptions as e:
                    msg = "{}, Retrying {} in {} seconds... {} tries left".format(e, f, mdelay, mtries)
                    if logger:
                        logger.warning(msg)
                    else:
                        print(msg)
                    time.sleep(mdelay)
                    mtries -= 1
                    mdelay *= backoff
            return f(*args, **kwargs)

        return f_retry  # true decorator

    return deco_retry


def stopwatch(f):
    @wraps(f)
    def _decorator_func(*args, **kwargs):
        start_time = int(round(time.time() * 1000))
        log = logging.getLogger(f.__module__)
        extra_args = {"func_name_override": f.__name__}
        return_val = f(*args, **kwargs)
        log.info(
            "{}() elapsed time: {} ms.".format(f.__name__, int(round(time.time() * 1000) - start_time)),
            extra=extra_args,
        )
        return return_val

    return _decorator_func



""" Log initializer """
from __future__ import absolute_import, division, print_function, unicode_literals

import itertools
import logging
import sys
from logging.handlers import RotatingFileHandler
from typing import List

DEBUG = logging.DEBUG
INFO = logging.INFO
ERROR = logging.ERROR
WARNING = logging.WARNING
DEFAULT_LOG_FILENAME = "superai.log"
_log_format = (
    "%(asctime)s - %(levelname)s - %(filename)s - %(threadName)s - [%(name)s:%(funcName)s:%(lineno)s] - %(message)s"
)
_date_format = "%Y-%m-%d %H:%M:%S"
_style = "{"

loggers: List[logging.Logger] = []


def create_file_handler(
    log_format=_log_format,
    log_filename=DEFAULT_LOG_FILENAME,
    max_bytes=5000000,
    backup_count=25,
):
    """ Create rotating file handler """
    formatter = CustomFormatter(fmt=log_format, datefmt=_date_format, style=_style)
    handler = RotatingFileHandler(log_filename, maxBytes=max_bytes, backupCount=backup_count)
    handler.setFormatter(formatter)
    return handler


def create_console_handler(log_format=_log_format, stream=sys.stdout):
    """ Create logging to stream """
    formatter = CustomFormatter(fmt=log_format, datefmt=_date_format)
    console_handler = logging.StreamHandler(stream)
    console_handler.setFormatter(formatter)
    return console_handler


def get_logger(name=None, propagate=True):
    """ Get logger object """
    logger = logging.getLogger(name)
    logger.propagate = propagate
    loggers.append(logger)
    return logger


def exception(line):
    """ Log exception """
    return logging.exception(line)


def debug(line):
    """ Log debug """
    return logging.debug(line)


def warn(line):
    """ Log warning """
    return logging.warn(line)


def error(line):
    """ Log error """
    return logging.error(line)


def info(line):
    """ Log info """
    return logging.info(line)


def init(filename=None, console=True, log_level=INFO, log_format=_log_format):
    """ Initialize logging setup """
    if not log_format:
        log_format = _log_format

    log_handlers: List[logging.Handler] = []
    if console:
        log_handlers.append(create_console_handler(log_format=log_format))
    if filename is not None:
        log_handlers.append(create_file_handler(log_format=log_format, log_filename=filename))

    for pair in itertools.product(loggers, log_handlers):
        pair[0].addHandler(pair[1])
        pair[0].setLevel(log_level)

    logging.basicConfig(format=_log_format, level=log_level, handlers=log_handlers)
    log = get_logger(__name__)
    if log_level > logging.INFO:
        log.log(level=log_level, msg=f"super.Ai logger initialized with log_level={log_level}")
    return log


class CustomFormatter(logging.Formatter):
    """Custom Formatter does these 2 things:
    1. Overrides 'funcName' with the value of 'func_name_override', if it exists.
    2. Overrides 'filename' with the value of 'file_name_override', if it exists.
    """

    def format(self, record):
        if hasattr(record, "func_name_override"):
            record.funcName = record.func_name_override
        if hasattr(record, "file_name_override"):
            record.filename = record.file_name_override
        if hasattr(record, "lineno_override"):
            record.lineno = record.lineno_override
        return super(CustomFormatter, self).format(record)


init()



from abc import ABC, abstractmethod
from datetime import datetime
from typing import Generator, List


class JobsApiMixin(ABC):
    @abstractmethod
    def request(self, uri, method, body_params=None, query_params=None, required_api_key=False):
        pass

    def create_jobs(
        self,
        app_id: str,
        callbackUrl: str = None,
        inputs: List[dict] = None,
        inputsFileUrl: str = None,
        metadata: dict = None,
    ) -> dict:
        """
        Submit jobs

        :param app_id: Application id
        :param callback_url: URL that should be POSTed once the job is completed for the response data.
        :param inputs: List of objects that represent the input of the job (based on the particular app type)
        :param inputsFileUrl: URL of json file containing list of input objects
        :param metadata: Object you can attach to job
        :return: Confirmation message of submission, batch id of submission, uuid of job if len(input) == 1
        """
        body_json = {}
        if callbackUrl is not None:
            body_json["callbackUrl"] = callbackUrl
        if inputs is not None:
            body_json["inputs"] = inputs
        if inputsFileUrl is not None:
            body_json["inputsFileUrl"] = inputsFileUrl
        if metadata is not None:
            body_json["metadata"] = metadata
        uri = f"apps/{app_id}/jobs"
        return self.request(uri, method="POST", body_params=body_json, required_api_key=True)

    def fetch_job(self, job_id: str) -> dict:
        """
        Get Job given job id

        :param job_id: Job id
        :return: Dict with job data
        """
        uri = f"jobs/{job_id}"
        return self.request(uri, method="GET", required_api_key=True)

    def fetch_batches_job(self, app_id) -> dict:
        """
        Get not processed Batches of submitted jobs given application id

        :param app_id: Application id
        :return: Dict with batch data
        """
        uri = f"apps/{app_id}/batches"
        return self.request(uri, method="GET", required_api_key=True)

    def fetch_batch_job(self, app_id: str, batch_id: str) -> dict:
        """
        Get Batch of submitted jobs given batch id and application id

        :param app_id: Application id
        :param batch_id: Batch id
        :return: Dict with batch data
        """
        uri = f"apps/{app_id}/batches/{batch_id}"
        return self.request(uri, method="GET", required_api_key=True)

    def get_job_response(self, job_id: str) -> dict:
        """
        Get Job Response given job id
        :param job_id:
        :return: Dict with job response
        """
        uri = f"jobs/{job_id}/response"
        return self.request(uri, method="GET", required_api_key=True)

    def cancel_job(self, job_id: str) -> dict:
        """
        Cancel a job given job id. Only for jobs in SCHEDULED, IN_PROGRESS or SUSPENDED state.

        :param job_id: Job id
        :return: Dict with job data
        """

        uri = f"jobs/{job_id}/cancel"
        return self.request(uri, method="POST", required_api_key=True)

    def review_job(
            self,
            job_id: str,
            correct: bool = None,
            response: dict = None,
    ) -> dict:
        """
        Review job
        :param job_id: Id or uuid of job
        :param correct: Mark job as correct if true, incorrect if false.
        :param response: Set job output and mark as correct.
        :return: Return dict with (job) response, correct and baselineId if review generated ground truth
        """
        body_json = {}
        if correct is not None:
            body_json["correct"] = correct
        if response is not None:
            body_json["response"] = response

        uri = f"/jobs/{job_id}/review"
        return self.request(uri, method="PATCH", body_params=body_json, required_api_key=True)

    def list_jobs(
        self,
        app_id: str,
        page: int = None,
        size: int = None,
        sortBy: str = "id",
        orderBy: str = "asc",
        createdStartDate: datetime = None,
        createdEndDate: datetime = None,
        completedStartDate: datetime = None,
        completedEndDate: datetime = None,
        statusIn: List[str] = None,
    ) -> dict:
        """
        Get a paginated list of jobs (without job responses) given an application id
        :param app_id: Application id
        :param page: Page number [0..N]
        :param size: Size of page
        :param sortBy: Job field to sort by
        :param orderBy: Sort direction (asc or desc)
        :param createdStartDate: Created start date
        :param createdEndDate: Created end date
        :param completedStartDate: Completed start date
        :param completedEndDate: Completed end date
        :param statusIn: Status of jobs
        :return: Paginated list of dicts with jobs data
        """
        uri = f"apps/{app_id}/jobs"
        query_params = {}
        if page is not None:
            query_params["page"] = page
        if size is not None:
            query_params["size"] = size
        if sortBy is not None:
            query_params["sortBy"] = sortBy
        if orderBy is not None:
            query_params["orderBy"] = orderBy
        if createdStartDate is not None:
            query_params["createdStartDate"] = createdStartDate.strftime("%Y-%m-%dT%H:%M:%SZ")
        if createdEndDate is not None:
            query_params["createdEndDate"] = createdEndDate.strftime("%Y-%m-%dT%H:%M:%SZ")
        if completedStartDate is not None:
            query_params["completedStartDate"] = completedStartDate.strftime("%Y-%m-%dT%H:%M:%SZ")
        if completedEndDate is not None:
            query_params["completedEndDate"] = completedEndDate.strftime("%Y-%m-%dT%H:%M:%SZ")
        if statusIn is not None:
            query_params["statusIn"] = statusIn
        return self.request(uri, method="GET", query_params=query_params, required_api_key=True)

    def download_jobs(
        self,
        app_id: str,
        createdStartDate: datetime = None,
        createdEndDate: datetime = None,
        completedStartDate: datetime = None,
        completedEndDate: datetime = None,
        statusIn: List[str] = None,
    ) -> str:
        """
        Trigger processing of jobs responses that is sent to customer email once is finished.
        :param app_id: Application id
        :param createdStartDate: Created start date
        :param createdEndDate: Created end date
        :param completedStartDate: Completed start date
        :param completedEndDate: Completed end date
        :param statusIn: Status of jobs
        :return: 'Task is processing' string
        """
        uri = f"apps/{app_id}/job_responses"
        query_params = {}
        if createdStartDate is not None:
            query_params["createdStartDate"] = createdStartDate.strftime("%Y-%m-%dT%H:%M:%SZ")
        if createdEndDate is not None:
            query_params["createdEndDate"] = createdEndDate.strftime("%Y-%m-%dT%H:%M:%SZ")
        if completedStartDate is not None:
            query_params["completedStartDate"] = completedStartDate.strftime("%Y-%m-%dT%H:%M:%SZ")
        if completedEndDate is not None:
            query_params["completedEndDate"] = completedEndDate.strftime("%Y-%m-%dT%H:%M:%SZ")
        if statusIn is not None:
            query_params["statusIn"] = statusIn
        return self.request(uri, method="POST", query_params=query_params, required_api_key=True)

    def get_all_jobs(
        self,
        app_id: str,
        sortBy: str = "id",
        orderBy: str = "asc",
        createdStartDate: datetime = None,
        createdEndDate: datetime = None,
        completedStartDate: datetime = None,
        completedEndDate: datetime = None,
        statusIn: List[str] = None,
    ) -> Generator[dict, None, None]:
        """
        Generator that retrieves all jobs (without job responses) given an application id
        :param app_id: Application id
        :param sortBy: Job field to sort by
        :param orderBy: Sort direction (asc or desc)
        :param createdStartDate: Created start date
        :param createdEndDate: Created end date
        :param completedStartDate: Completed start date
        :param completedEndDate: Completed end date
        :param statusIn: Status of jobs
        :return: Generator that yields complete list of dicts with jobs data
        """
        page = 0
        paginated_jobs = {"pages": 1}
        while page <= paginated_jobs["pages"] - 1:
            paginated_jobs = self.list_jobs(
                app_id,
                page=page,
                size=500,
                sortBy=sortBy,
                orderBy=orderBy,
                createdStartDate=createdStartDate,
                createdEndDate=createdEndDate,
                completedStartDate=completedStartDate,
                completedEndDate=completedEndDate,
                statusIn=statusIn,
            )
            for job in paginated_jobs["jobs"]:
                yield job
            page = page + 1



# coding: utf-8


from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

NAME = "superai"
VERSION = "0.0.9"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = [
    "names",  # TODO: Remove in next versions
    "boto3>=1.16",
    "click>=7.0",
    "colorama>=0.4.3",
    "dynaconf>=3.1.2",
    "futures-then>=0.1.1",
    "genson>=1.2.2",
    "jinja2>=2.11.2",
    "joblib>=0.17.0",
    "jsonmerge~=1.7.0",
    "jsonpickle>=1.4.1",
    "pyyaml>=5.3.1",
    "requests>=2.22",
    "sentry-sdk>=0.19.4",
    "scikit-learn>=0.23.2",
    "warrant>=0.6",
]

BUILD_REQUIRES = [
    "awscli>=1.18.163",
    "bump2version~=1.0.1",
    "setuptools>=50.3.2",
    "Sphinx>=3.2.1",
    "wheel>=0.35.1",
]
DEV_REQUIRES = [
]
TEST_REQUIRES = [
    "deepdiff>=4.0.7",
    "tox>=2.9.1",
    "coverage>=5.3",
    "pytest>=6.1.2",
    "pytest-cov>=2.10.1",
    "pytest-env>=0.6.2",
]

setup(
    name=NAME,
    version=VERSION,
    description="super.AI API",
    author="super.AI",
    author_email="support@super.ai",
    url="https://github.com/mysuperai/superai-api-client",
    keywords=["super.AI API"],
    install_requires=REQUIRES,
    extras_require={
        "build": BUILD_REQUIRES,
        "dev": DEV_REQUIRES,
        "test": TEST_REQUIRES,
    },
    packages=find_packages(),
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    entry_points={
        "console_scripts": [
            "superai=superai.cli:main",
        ],
    },
)



import requests
from typing import Optional

from superai.apis.auth import AuthApiMixin
from superai.apis.data import DataApiMixin
from superai.apis.ground_truth import GroundTruthApiMixin
from superai.apis.jobs import JobsApiMixin
from superai.config import settings
from superai.exceptions import SuperAIError

BASE_URL = settings.get("base_url")


class Client(JobsApiMixin, AuthApiMixin, GroundTruthApiMixin, DataApiMixin):
    def __init__(self, api_key: str = None, auth_token: str = None, base_url: str = None):
        self.api_key = api_key
        self.auth_token = auth_token
        if base_url is None:
            self.base_url = BASE_URL
        else:
            self.base_url = base_url

    def request(
        self,
        endpoint: str,
        method: str = "GET",
        query_params: dict = None,
        body_params: dict = None,
        required_api_key: bool = False,
        required_auth_token: bool = False,
    ) -> Optional[dict]:
        headers = {}
        if required_api_key and self.api_key:
            headers["API-KEY"] = self.api_key
        if required_auth_token and self.auth_token:
            headers["AUTH-TOKEN"] = self.auth_token
        resp = requests.request(
            method, f"{self.base_url}/{endpoint}", params=query_params, json=body_params, headers=headers
        )
        try:
            resp.raise_for_status()
            if resp.status_code == 204:
                return None
            else:
                return resp.json()
        except requests.exceptions.HTTPError as http_e:
            try:
                message = http_e.response.json()["message"]
            except:
                message = http_e.response.text
            raise SuperAIError(message, http_e.response.status_code)



import logging
from unittest import TestCase
from unittest.mock import Mock

from superai.log.decorator import log_exception, log_on_end, log_on_error, log_on_start


def func_test(arg1, arg2, kwarg1=None, kwarg2=None):
    return arg1 + arg2


class TestException(Exception):
    pass


class MockLoggingHandler(logging.Handler):
    """Mock logging handler to check for expected logs.

    Messages are available from an instance's ``messages`` dict, in order, indexed by
    a lowercase log level string (e.g., 'debug', 'info', etc.).
    """

    def __init__(self, *args, **kwargs):
        self.messages = {"debug": [], "info": [], "warning": [], "error": [], "critical": []}
        super(MockLoggingHandler, self).__init__(*args, **kwargs)

    def emit(self, record):
        "Store a message from ``record`` in the instance's ``messages`` dict."
        try:
            self.messages[record.levelname.lower()].append(record.getMessage())
        except Exception:
            self.handleError(record)

    def reset(self):
        self.acquire()
        try:
            for message_list in self.messages.values():
                message_list.clear()
        finally:
            self.release()


class TestDecorators(TestCase):
    def setUp(self):
        self.logger = logging.Logger("mocked")
        self.log_handler = MockLoggingHandler()
        self.log_handler.setFormatter("%(msg)s")
        self.logger.addHandler(self.log_handler)

    def test_log_on_start(self):
        dec = log_on_start(logging.INFO, "test message {arg1:d}, {arg2:d}", logger=self.logger)
        fn = dec(func_test)
        fn(1, 2)
        self.assertIn("test message 1, 2", self.log_handler.messages["info"])

    def test_log_on_end(self):
        dec = log_on_end(logging.INFO, "test message {arg1:d}, {arg2:d} => {result:d}", logger=self.logger)
        fn = dec(func_test)
        fn(1, 2)
        self.assertIn("test message 1, 2 => 3", self.log_handler.messages["info"])

    def test_log_on_error(self):
        mocked_func = Mock(side_effect=TestException("test exception"))

        dec = log_on_error(logging.INFO, "test message {e!r}", logger=self.logger, on_exceptions=TestException)
        fn = dec(mocked_func)
        fn(1, 2)
        self.assertIn("test message TestException('test exception')", self.log_handler.messages["info"])

    def test_log_on_error_reraise(self):
        mocked_func = Mock(side_effect=TestException("test exception"))

        dec = log_on_error(
            logging.INFO, "test message {e!r}", logger=self.logger, on_exceptions=TestException, reraise=True
        )
        fn = dec(mocked_func)

        with self.assertRaises(TestException):
            fn(1, 2)

        self.assertIn("test message TestException('test exception')", self.log_handler.messages["info"])

    def test_log_exception(self):
        self.logger.exception = Mock()
        dec = log_exception(logging.ERROR, "test message", logger=self.logger, on_exceptions=TypeError)
        fn = dec(func_test)
        fn(2, "asd")
        self.assertEqual(self.logger.exception.call_count, 1)
